@@ -161,7 +161,7 @@ function (x,
                         )
                 scale <- rep(FALSE, ncol(x))
             } else {
-                xtmp <- scale(x[,scale])
+                xtmp <- scale_data_frame(x[,scale])
                 x[,scale] <- xtmp
                 x.scale <- attributes(xtmp)[c("scaled:center","scaled:scale")]
                 if (is.numeric(y) && (type > 2)) {
@@ -419,6 +419,10 @@ function (object, newdata,
             newdata <- model.matrix(delete.response(terms(object)),
                                     as.data.frame(newdata))
         } else {
+            ## FIXME: would be safer, but users might provide new data with
+            ## other colnames than the training data ...
+            ## if (!is.null(colnames(newdata)))
+            ##    newdata <- newdata[,colnames(object$SV)]
             newdata <- na.action(as.matrix(newdata))
             act <- attr(newdata, "na.action")
         }
@@ -427,16 +431,16 @@ function (object, newdata,
     if (!is.null(act) && !preprocessed)
         rowns <- rowns[-act]
 
-    if (any(object$scaled))
-        newdata[,object$scaled] <-
-            scale(newdata[,object$scaled, drop = FALSE],
-                  center = object$x.scale$"scaled:center",
-                  scale  = object$x.scale$"scaled:scale"
-                  )
-
     if (ncol(object$SV) != ncol(newdata))
         stop ("test data does not match model !")
 
+    if (any(object$scaled))
+        newdata[,object$scaled] <-
+            scale_data_frame(newdata[,object$scaled, drop = FALSE],
+                  center = object$x.scale$"scaled:center",
+                  scale  = object$x.scale$"scaled:scale"
+            )
+
     ret <- .C (R_svmpredict,
                as.integer (decision.values),
                as.integer (probability),
@@ -587,20 +591,6 @@ function (x, ...)
     cat("\n\n")
 }
 
-scale.data.frame <-
-function(x, center = TRUE, scale = TRUE)
-{
-    i <- sapply(x, is.numeric)
-    if (ncol(x[, i, drop = FALSE])) {
-        x[, i] <- tmp <- scale.default(x[, i, drop = FALSE], na.omit(center), na.omit(scale))
-        if(center || !is.logical(center))
-            attr(x, "scaled:center")[i] <- attr(tmp, "scaled:center")
-        if(scale || !is.logical(scale))
-            attr(x, "scaled:scale")[i]  <- attr(tmp, "scaled:scale")
-    }
-    x
-}
-
 plot.svm <-
 function(x, data, formula = NULL, fill = TRUE,
          grid = 50, slice = list(), symbolPalette = palette(),
@@ -615,8 +605,8 @@ function(x, data, formula = NULL, fill = TRUE,
             stop("missing formula.")
         if (fill) {
             sub <- model.frame(formula, data)
-            xr <- seq(min(sub[, 2]), max(sub[, 2]), length = grid)
-            yr <- seq(min(sub[, 1]), max(sub[, 1]), length = grid)
+            xr <- seq(min(sub[, 2]), max(sub[, 2]), length.out = grid)
+            yr <- seq(min(sub[, 1]), max(sub[, 1]), length.out = grid)
             l <- length(slice)
             if (l < ncol(data) - 3) {
                 slnames <- names(slice)
@@ -626,7 +616,7 @@ function(x, data, formula = NULL, fill = TRUE,
                 names(slice) <- c(slnames, names[!names %in%
                                                  c(colnames(sub), slnames)])
             }
-            for (i in names(which(sapply(data, is.factor))))
+            for (i in names(which(vapply(data, is.factor, NA))))
                 if (!is.factor(slice[[i]])) {
                     levs <- levels(data[[i]])
                     lev <- if (is.character(slice[[i]])) slice[[i]] else levs[1]
@@ -726,7 +716,11 @@ coef.svm <- function(object, ...)
 {
     if (object$kernel != 0 || object$nclasses > 2)
         stop("Only implemented for regression or binary classification with linear kernel.")
-    ret <- drop(crossprod(object$coefs, object$SV))
+    ret <- if (inherits(object$SV, "matrix.csr")) {
+        loadNamespace("SparseM")
+        drop(crossprod(object$coefs, SparseM::as.matrix(object$SV)))
+    } else
+        drop(crossprod(object$coefs, object$SV))
     trm <- object$terms
     if(!is.null(trm))
         names(ret) <- labels(trm)
